/* -*-c-*-
 *
 * $Id: factorial.c,v 1.2 2001/06/16 13:22:59 mdw Exp $
 *
 * Example factorial computation
 *
 * (c) 2000 Straylight/Edgeware
 */

/*----- Licensing notice --------------------------------------------------* 
 *
 * This file is part of Catacomb.
 *
 * Catacomb is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Library General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 * 
 * Catacomb is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Library General Public License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with Catacomb; if not, write to the Free
 * Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 * MA 02111-1307, USA.
 */

/*----- Revision history --------------------------------------------------* 
 *
 * $Log: factorial.c,v $
 * Revision 1.2  2001/06/16 13:22:59  mdw
 * Added command-line option to select output radix.
 *
 * Revision 1.1  2000/07/09 21:30:49  mdw
 * Demo program to compute factorials.
 *
 */

/*----- Header files ------------------------------------------------------*/

#include <stdio.h>
#include <stdlib.h>

#include <mLib/mdwopt.h>
#include <mLib/quis.h>
#include <mLib/report.h>

#include "mpmul.h"

/*----- Main code ---------------------------------------------------------*/

int main(int argc, char *argv[])
{
  unsigned long x;
  int r = 10;
  char *p;
  mp *f;

  ego(argv[0]);

  for (;;) {
    static const struct option opt[] = {
      { "radix",	OPTF_ARGREQ,	0,	'r' },
      { 0,		0,		0,	0 }
    };
    int i = mdwopt(argc, argv, "r:", opt, 0, 0, 0);
    if (i < 0)
      break;
    switch (i) {
      case 'r':
	r = atoi(optarg);
	if (r < 2 || r > 36)
	  die(EXIT_FAILURE, "bad radix `%s'", optarg);
	break;
      default:
	exit(EXIT_FAILURE);
    }
  }

  if (optind + 1 != argc) {
    pquis(stderr, "Usage: $ [-r radix] integer\n");
    exit(EXIT_FAILURE);
  }
  x = strtoul(argv[optind], &p, 0);
  if (*p)
    die(EXIT_FAILURE, "bad integer `%s'", argv[optind]);
  f = mp_factorial(x);
  mp_writefile(f, stdout, r);
  fputc('\n', stdout);
  mp_drop(f);
  return (0);
}

/*----- That's all, folks -------------------------------------------------*/
